@@ -1,11 +1,13 @@
 # -*- coding: utf-8 -*-
 
 import requests
+import grequests
 import uuid
 import copy
 import os
 import urlparse
 import json
+import gevent
 
 from .patterns import Regex
 from .responses import ( DoneLoad, DoneFind, Wait, MissingTags,
@@ -137,7 +139,11 @@ class Scraper(object):
             return Failed(req, "'%s' failed because of %s" % (instruction['find'],
                                                               str(e)))
 
-        results = []
+        if len(subs) == 0:
+            return Failed(req, "No matches for '%s', evaluated to '%s'" % (
+                instruction['find'], findSub.result))
+
+        greenlets = []
         # Call children once for each substitution, using it as input
         # and with a modified set of tags.
         for s in subs:
@@ -148,18 +154,21 @@ class Scraper(object):
 
             fork_tags = copy.deepcopy(req.tags)
             fork_tags[name] = s
-            results.append(Result(s,
-                                  Scraper(session=self._session, force_all=self._force_all).scrape(then,
-                                                                tags=fork_tags,
-                                                                input=s,
-                                                                uri=req.uri
-                                                               )))
+            child_scraper = Scraper(session=self._session, force_all=self._force_all)
 
-        if len(results):
-            return DoneFind(req, name, description, results)
-        else:
-            return Failed(req, "No matches for '%s', evaluated to '%s'" % (
-                instruction['find'], findSub.result))
+            greenlets.append(child_scraper.scrape_async(then,
+                                                        tags=fork_tags,
+                                                        input=s,
+                                                        uri=req.uri))
+
+        gevent.joinall(greenlets)
+
+        # Build Results with responses from greenlets
+        results = []
+        for i, s in enumerate(subs):
+            results.append(Result(s, greenlets[i].get()))
+
+        return DoneFind(req, name, description, results)
 
     def _scrape_load(self, req, instruction, description, then):
         """
@@ -197,23 +206,27 @@ class Scraper(object):
         headers = headersSub.result
 
         try:
-            opts = dict(cookies=cookies, headers=headers)
+            opts = dict(cookies=cookies,
+                        headers=headers,
+                        session=self._session)
             if method == 'post' or posts:
                 opts['data'] = posts
                 # Force use of POST if post-data was set.
                 method = 'post'
 
-            requester = getattr(self._session, method)
+            requester = getattr(grequests, method)
 
-            resp = requester(urlSub.result, **opts)
+            greq = requester(urlSub.result, **opts)
+            greq.send()
+            resp = greq.response
             if resp.status_code == 200:
                 # Call children using the response text as input
-                result = Result(resp.text,
-                                Scraper(session=self._session, force_all=self._force_all).scrape(then,
-                                                              tags=req.tags,
-                                                              input=resp.text,
-                                                              uri=req.uri
-                                                             ))
+                child_scraper = Scraper(session=self._session, force_all=self._force_all)
+                scraper_results = child_scraper.scrape(then,
+                                                       tags=req.tags,
+                                                       input=resp.text,
+                                                       uri=req.uri)
+                result = Result(resp.text, scraper_results)
                 return DoneLoad(req, name, description, result, resp.cookies)
             else:
                 return Failed(req, "Status code %s from %s" % (
@@ -349,12 +362,16 @@ class Scraper(object):
 
         # Handle each element of list separately within this context.
         if isinstance(instruction, list):
-            return map(lambda i: Scraper(session=self._session, force_all=self._force_all).scrape(i,
-                                                               tags=tags,
-                                                               input=input,
-                                                               force=force,
-                                                               uri=uri),
-                      instruction)
+            greenlets = map(lambda i: Scraper(session=self._session,
+                                              force_all=self._force_all
+                                             ).scrape_async(i,
+                                                            tags=tags,
+                                                            input=input,
+                                                            force=force,
+                                                            uri=uri),
+                            instruction)
+            gevent.joinall(greenlets)
+            return [g.get() for g in greenlets]
 
         # Dict instructions are ones we can actually handle
         elif isinstance(instruction, dict):
@@ -363,3 +380,10 @@ class Scraper(object):
         # Fail.
         else:
             raise InvalidInstructionError(instruction)
+
+    def scrape_async(self, instruction, tags={}, input='', force=False, **kwargs):
+        """
+        Scrape a request like `scrape`, except returns a greenlet which
+        supplies the Response or list of Responses from `get`.
+        """
+        return gevent.spawn(self.scrape, instruction, tags, input, force=False, **kwargs)
